from logilab.mtconverter import xml_escape

from cubicweb.web import component
from cubicweb.selectors import has_related_entities, is_instance

# XXX this component looks generic... move it to cubicweb ?
class ExternalUriComponent(component.EntityCtxComponent):
    """component to display link to external uri"""
    __regid__ = 'externaluricomp'
    __select__ = (component.EntityCtxComponent.__select__
                  & has_related_entities('same_as')
                  & ~is_instance('ExternalUri'))
    context = 'ctxtoolbar'

    def render_body(self, row, col, view=None):
        self.w(u'<div class="toolbarButton">')
        imgurl = self._cw.uiprops['BLUE_ARROW']
        if hasattr(self.entity, 'url'):
            url = self.entity.url
        else:
            url = self.entity.same_as[0].uri
        self.w(u'<a href="%s"><img src="%s" alt="blue arrow"/></a>'
               % (xml_escape(url), xml_escape(imgurl)))
        self.w(u'</div>')



# XML <-> yams equivalence
from cubicweb.xy import xy
xy.register_prefix('bibo', 'http://purl.org/ontology/bibo/')

xy.add_equivalence('Book', 'bibo:Book')
xy.add_equivalence('Book title', 'bibo:Book dc:title')
xy.add_equivalence('Book publish_date', 'bibo:Book dc:date')
xy.add_equivalence('Book isbn10', 'bibo:Book bibo:isbn10')
xy.add_equivalence('Book isbn13', 'bibo:Book bibo:isbn13')



"""template automatic tests"""

from logilab.common.testlib import unittest_main
from cubicweb.devtools.testlib import AutomaticWebTest
from cubicweb.devtools.fill import ValueGenerator

import random

def random_numbers(size):
    return u''.join(random.choice('0123456789') for i in xrange(size))

class MyValueGenerator(ValueGenerator):
    def generate_Book_isbn10(self, entity, index):
        return random_numbers(10)
    def generate_Book_isbn13(self, entity, index):
        return random_numbers(13)

class AutomaticWebTest(AutomaticWebTest):

    def to_test_etypes(self):
        return set(('Book', 'Collection', 'Editor'))

    def list_startup_views(self):
        return ()

if __name__ == '__main__':
    unittest_main()



"""
Simple API for OpenLibrary <http://openlibrary.org/dev/docs>

Try running::

  $ python olapi.py search tom sawyer

:organization: Logilab
:copyright: 2009-2017 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from urllib2 import urlopen, HTTPError, URLError
from urllib import urlencode
from json import loads, dumps

class OpenLibraryFailure(Exception):
    """raised on API failure"""

def openlibrary_fetch(url):
    try:
        result = loads(urlopen(url).read()) # XXX handle timeouts
    except URLError, exc:
        raise OpenLibraryFailure(str(exc))
    # depending on the type of request, result is not identically structured
    # result from a search will have a status key, otherwise result is the
    # set of result itself
    if 'result' in result:
        if result[u'status'] != u'ok':
            raise OpenLibraryFailure(result[u'message'])
        return result[u'result']
    else:
        return result

def openlibrary_query(type, **kwargs):
    query = dict(type=type, **kwargs)
    # required parameter * empty to collect all properties from books
    # objects
    query['*'] = ''
    url = 'http://openlibrary.org/query.json?%s' % urlencode(query)
    return openlibrary_fetch(url)

def search_books(phrase):
    query = dict(query=phrase)
    url = 'http://openlibrary.org/api/search?%s' % urlencode({'q': dumps(query)})
    return openlibrary_fetch(url)

def get_bookcover(coverid, size='M'):
    try:
        url = 'http://covers.openlibrary.org/w/id/%s-%s.jpg' % (coverid, size)
        data = urlopen(url).read()
    except HTTPError, exc:
        raise OpenLibraryFailure(str(exc))
    return data

def get_authors(bookinfo):
    authors = []
    for author_key in bookinfo.get('authors', []):
        authinfo = openlibrary_query('/type/author', **author_key)
        authors += authinfo
    return authors


def set_bookinfo_cover(bookinfo, cover_required=False):
    coverids = bookinfo.get('covers', [])
    for coverid in coverids:
        try:
            bookinfo['cover'] = get_bookcover(coverid)
        except OpenLibraryFailure, exc:
            if cover_required:
                raise exc

def get_bookinfo(cover_required=False, **kwargs):
    infos = openlibrary_query('/type/edition', **kwargs)
    if infos:
        bookinfo = infos[0]
        set_bookinfo_cover(bookinfo, cover_required)
        return bookinfo
    return {}

if __name__ == '__main__':
    import sys, pprint
    if sys.argv[1] == 'search':
        pprint.pprint(search_books(sys.argv[2]))



from yams.buildobjs import (EntityType, SubjectRelation, RelationDefinition,
                            String, Int, Date)

from cubes.person.schema import Person

_ = unicode


Person.add_relation(SubjectRelation('ExternalUri', cardinality='**'), name='same_as')

class Book(EntityType):
    title = String(required=False, # handled by hook
                   fulltextindexed=True, indexed=True,
                   maxsize=255, description=_('book title'))
    subtitle = String(fulltextindexed = True, maxsize=255, description = _('book subtitle'))
    isbn10 = String(unique=True, fulltextindexed=True, indexed=True,
                    maxsize=10, description=_('Old International Standard Book Number (10 chars, no spaces or dashes)'))
    isbn13 = String(unique=True, fulltextindexed=True, indexed=True,
                    maxsize=13, description=_('New International Standard Book Number (13 chars, no spaces or dashes)'))
    pages = Int(description=_('number of pages'))
    publish_date = Date(description=_('publication date'))
    language = String(fulltextindexed=True, indexed=True, maxsize=50, description=_('language of the book'))
    summary = String(fulltextindexed=True, indexed=True,
                     description=_('Short summary of the book'))
    content = String(fulltextindexed=True, indexed=True, description=_('table of contents'))
    # relations
    authors = SubjectRelation('Person', cardinality='**', description=_('book authors'))
    publisher = SubjectRelation('Editor', cardinality='**', description=_('book publisher'))
    editor = SubjectRelation('Person', cardinality='**', description=_('book editor'))
    collection = SubjectRelation('Collection', cardinality='*1', description=_('publisher\'s collection'))
    has_cover = SubjectRelation('File', cardinality='**', composite='subject', description=_('book cover'))
    same_as = SubjectRelation('ExternalUri', cardinality='**', composite='subject')

class Collection(EntityType):
    name = String(required=True, fulltextindexed=True, indexed=True, maxsize=100)

class has_collection(RelationDefinition):
    subject = 'Editor'
    object = 'Collection'
    cardinality = '**'

class Editor(EntityType): # XXX rename to Publisher
    name = String(required=True, fulltextindexed=True, indexed=True, maxsize=100)
    has_address = SubjectRelation('PostalAddress', cardinality='*1')

class same_as(RelationDefinition):
    subject = 'Person'
    object = 'ExternalUri'
    cardinality = '**'
    composite = 'subject'



"""
To use this script:
cubicweb-ctl shell appid import_freebase.py number_of_books_to_import

number_of_books_to_import is optionnal, if not provided, all books
will be imported
"""

import freebase

from cubicweb.dataimport import CWImportController, RQLObjectStore

from cubes.book.migration.import_ol import gen_books, to_date
from cubes.book.migration.import_dbpedia import set_isbn, set_pages

def get_books(limit):
    query = {'type': '/book/book_edition', '*': None, 'binding': 'hardcover'}
    if limit:
        query['limit'] = int(limit)
    results = freebase.mqlreaditer(query)
    for book in results:
        try:
            bookinfo = {}
            bookinfo['title'] = unicode(book['book'])
            bookinfo['publish_date'] = to_date(book['publication_date'])
            bookinfo['cwuri'] = u'http://www.freebase.com'
            bookinfo['externaluri'] = unicode('%s/view%s' % (bookinfo['cwuri'], book['id']))
            if book.get('isbn'):
                bookinfo['isbn'] = book['isbn']
            if book.get('number_of_pages'):
                bookinfo['pages'] = book['number_of_pages']
                set_pages(bookinfo)
            bookinfo['authors_name'] = book['author_editor']
            if book.get('publisher'):
                print "publisher", book.get('publisher')
                bookinfo['publisher'] = unicode(book['publisher'])
                print "after"
            set_authors(bookinfo)
            set_isbn(bookinfo)
            yield bookinfo
        except Exception, e:
            print "EXCEPTION in get_books:", e

def set_authors(bookinfo):
    for name in bookinfo['authors_name']:
        query = {'type': '/book/author', 'name': name, '*': None, 'book_editions_published': bookinfo['title']}
        author = freebase.mqlread(query)
        if author:
            author_infos = {}
            author_infos['name'] = unicode(author['name'])
            author_infos['cwuri'] = u'http://www.freebase.com'
            author_infos['externaluri'] = unicode('%s/view%s' % (author_infos['cwuri'], author['id']))
            bookinfo.setdefault('authors', [])
            bookinfo['authors'].append(author_infos)



GENERATORS = []
CHK = []
GENERATORS.append( (gen_books, CHK) )
if __name__ == '__main__':
    import sys
    try:
        limit = sys.argv[4]
    except:
        limit = None

    # create controller
    ctl = CWImportController(RQLObjectStore(session))
    ctl.askerror = 1
    ctl.generators = GENERATORS
    ctl.data['books'] = get_books(limit)
    # run
    ctl.run()



"""
to use this script:
cubicweb-ctl shell appid import_ol.py keywords to_search in_openlibrary
"""

from cubes.book.olapi import search_books
from cubicweb.dataimport import CWImportController, RQLObjectStore, mk_entity
from datetime import datetime
from functools import partial
from lxml import etree

NAMESPACES = {'rdf': 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
              'bibo': 'http://purl.org/ontology/bibo/',
              'dcterms': 'http://purl.org/dc/terms/',
              'ol': 'http://openlibrary.org/type/author'}

def get_node_value(tree, node_name):
    node = tree.xpath('/rdf:RDF/rdf:Description/%s' % node_name,
                      namespaces=NAMESPACES)
    if len(node) > 0:
        return unicode(node[0].text)
    return u''

def get_externaluri(tree):
    externaluri = tree.xpath('/rdf:RDF/rdf:Description[@rdf:about]',
                             namespaces=NAMESPACES)
    if len(externaluri) > 0:
        return unicode(externaluri[0].get('{%s}about' % NAMESPACES['rdf']))
    return u''

def get_authors(tree):
    authors = tree.xpath('/rdf:RDF/rdf:Description/bibo:authorList/rdf:Description[@rdf:about]',
                         namespaces=NAMESPACES)
    authors_name = tree.xpath('/rdf:RDF/rdf:Description/bibo:authorList/rdf:Description/rdf:value',
                         namespaces=NAMESPACES)
    if len(authors) > 0:
        authors_infos = []
        for (author, author_name) in zip(authors, authors_name):
            item = {}
            item['externaluri'] = unicode(author.get('{%s}about' % NAMESPACES['rdf']))
            item['cwuri'] = unicode('http://openlibrary.org')
            item['name'] = unicode(author_name.text)
            authors_infos.append(item)
        return authors_infos
    return []

def uget(node, attr):
    value = node.get(attr)
    if isinstance(value, str):
        # if lxml returns a str value, it's a pure-ascii string
        value = unicode(value)
    return value

def to_date(date):
    if ',' in date:
        return datetime.strptime(date, '%B %d, %Y')
    elif date.isdigit():
        return datetime(int(date), 1, 1)
    raise ValueError('unknown date format')

def fetch_book_rdf(bookid):
    """Work around bugs in OL's RDF"""
    import urllib2
    bookid = bookid.replace('/b/','/books/')
    url = 'http://openlibrary.org%s.rdf' % bookid
    print '----------',url
    data = urllib2.urlopen(url).read()
    try:
        tree = etree.fromstring(data)
    except Exception, exc:
        print 'Trying to work around', exc
        data = data.replace('</rdf:RDF>','</rdf:Description></rdf:RDF>')
        tree = etree.fromstring(data)
    return tree

def get_books(phrase):
    books = search_books(phrase)
    print "total, books", len(books)
    for book in books:
        if not book:
            print 'empty book', book
            continue
        try:
            tree = fetch_book_rdf(book)
        except Exception, exc:
            print 'Failed. Ignoring this book', exc
        bookinfo = {}
        bookinfo['isbn10'] = get_node_value(tree, 'bibo:isbn10')
        bookinfo['isbn13'] = get_node_value(tree, 'bibo:isbn13')
        bookinfo['cwuri'] = u'http://openlibrary.org'
        bookinfo['externaluri'] = get_externaluri(tree)
        bookinfo['title'] = get_node_value(tree, 'dcterms:title')[:128]
        bookinfo['publisher'] = get_node_value(tree, 'dcterms:publisher')
        try:
            bookinfo['publish_date'] = to_date(get_node_value(tree, 'dcterms:issued'))
        except Exception, exc:
            print 'Failed to read publish_date because', exc
            bookinfo['publish_date'] = None
        # authors is a list of urls leading to author external uri
        # from which we'll parse rdf content
        bookinfo['authors'] = get_authors(tree)
        yield bookinfo

def remove_book_same_id(id_type, id_value):
    beid = already_exists(ctl, 'Book', id_type, id_value)
    if beid:
        ctl.store.rql('DELETE Book B WHERE B eid %s' % beid)

def already_exists(ctl, etype, attrname, attrvalue):
    rql = 'Any X WHERE X is %s, X %s "%s"' % (etype, attrname, attrvalue)
    rset = ctl.store.rql(rql)
    if rset:
        return rset[0][0]
    return None

def already_exists_relation(ctl, seid, rtype, oeid):
    rql = 'Any X, Y WHERE X %s Y, X eid %s, Y eid %s' % (rtype, seid, oeid)
    rset = ctl.store.rql(rql)
    if rset:
        return True
    return False

# XXX move this to cw.dataimport ?
def _maxsize(size, text):
    if len(text) > size:
        print 'Truncating from', len(text), 'to size', size, text
    return text[:size]

def maxsize(size):
    return partial(_maxsize, size)

GENERATORS = []

BOOKS = [('title', 'title', (maxsize(128),)),
         ('cwuri', 'cwuri', ()),
         ]

AUTHORS = [('name', 'surname', (maxsize(64),))]
PUBLISHERS = [('name', 'name', (maxsize(100),))]
CHK = []

def gen_books(ctl):
    for bookinfo in ctl.get_data('books'):
        if already_exists(ctl, 'Book', 'isbn10', bookinfo['isbn10']) or \
            already_exists(ctl, 'Book', 'isbn13', bookinfo['isbn13']):
            beid = already_exists(ctl, 'Book', 'isbn10', bookinfo['isbn10']) or \
                    already_exists(ctl, 'Book', 'isbn13', bookinfo['isbn13'])
            book = ctl.store.rql('Any B WHERE B is Book, B eid %s' % beid).get_entity(0,0)
            # update exsting book with new information
            book.update(bookinfo)
        else:
            entity = mk_entity(bookinfo, BOOKS)
            if bookinfo['isbn10']:
                entity['isbn10'] = bookinfo['isbn10']
            if bookinfo['isbn13']:
                entity['isbn13'] = bookinfo['isbn13']
            if 'pages' in bookinfo:
                if bookinfo['pages']:
                    entity['pages'] = bookinfo['pages']
            ctl.store.add('Book', entity)
            beid = entity['eid']
        # XXX cf datafeed.sget_externaluri()
        if not already_exists(ctl, 'ExternalUri', 'uri', bookinfo['externaluri']):
            externaluri = mk_entity({'uri': bookinfo['externaluri']}, [('uri', 'uri', ()),])
            ctl.store.add('ExternalUri', externaluri)
            eueid = externaluri['eid']
        else:
            eueid = already_exists(ctl, 'ExternalUri', 'uri', bookinfo['externaluri'])
        if not already_exists_relation(ctl, beid, 'same_as', eueid):
            ctl.store.relate(beid, 'same_as', eueid)
        if 'authors' in bookinfo:
            for authorinfo in bookinfo.get('authors'):
                try:
                    # XXX ideally we want to collect author information from url
                    # but since RDF for author is invalid for now, we can not use it
                    # we have to wait until this is fixed.
                    # get author information from RDf source
                    # url = u'%s.rdf' % author
                    # tree = etree.parse(url)
                    if not already_exists(ctl, 'Person', 'surname', authorinfo['name']):
                        author = mk_entity(authorinfo, AUTHORS)
                        ctl.store.add('Person', author)
                        aeid = author['eid']
                    else:
                        aeid = already_exists(ctl, 'Person', 'surname', authorinfo['name'])
                    if not already_exists_relation(ctl, beid, 'authors', aeid):
                        ctl.store.relate(beid, 'authors', aeid)
                    # Handle external URI
                    if not already_exists(ctl, 'ExternalUri', 'uri', authorinfo['externaluri']):
                        externaluri = mk_entity({'uri': authorinfo['externaluri']}, [('uri', 'uri', ()),])
                        ctl.store.add('ExternalUri', externaluri)
                        eueid = externaluri['eid']
                    else:
                        eueid = already_exists(ctl, 'ExternalUri', 'uri',
                                               authorinfo['externaluri'])
                    if not already_exists_relation(ctl, aeid, 'same_as', eueid):
                        ctl.store.relate(aeid, 'same_as', eueid)

                except Exception, e:
                    print "EXCEPTION:", e
                    break
        if 'publisher' in bookinfo:
            if not already_exists(ctl, 'Editor', 'name', bookinfo['publisher']):
                editorinfo = {'name': bookinfo['publisher']}
                editor = mk_entity(editorinfo, PUBLISHERS)
                ctl.store.add('Editor', editor)
                eeid = editor['eid']
            else:
                eeid = already_exists(ctl, 'Editor', 'name', bookinfo['publisher'])
            if not already_exists_relation(ctl, beid, 'publisher', eeid):
                ctl.store.relate(beid, 'publisher', eeid)
        ctl.store.commit()

GENERATORS.append( (gen_books, CHK) )

if __name__ == '__main__':
    import sys
    phrase = ' '.join(sys.argv[4:])
    # define data generators

    # create controller
    ctl = CWImportController(RQLObjectStore(session))
    ctl.askerror = 1
    ctl.generators = GENERATORS
    ctl.data['books'] = get_books(phrase)
    # run
    ctl.run()



from cubicweb.entities import AnyEntity, fetch_config
from cubicweb.view import EntityAdapter
from cubicweb.selectors import is_instance

from cubes.person.entities import Person

from lxml import etree

NAMESPACES = {'res': "http://www.w3.org/2005/sparql-results#",
              'dbpprop': "http://dbpedia.org/property/",
              'foaf': "http://xmlns.com/foaf/0.1/",
              'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
              'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
              'dbpedia-owl': "http://dbpedia.org/ontology/"}

class Book(AnyEntity):
    __regid__ = 'Book'
    fetch_attrs, cw_fetch_order = fetch_config(['title', 'isbn13', 'isbn10', 'pages'])



class BookICalendarableAdapter(EntityAdapter):
    __regid__ = 'ICalendarable'
    __select__ = is_instance('Book')

    @property
    def start(self):
        return self.entity.publish_date
    stop = start

class Collection(AnyEntity):
    __regid__ = 'Collection'

    fetch_attrs, cw_fetch_order = fetch_config(['name'])

class Editor(AnyEntity):
    __regid__ = 'Editor'

    fetch_attrs, cw_fetch_order = fetch_config(['name'])

class Author(Person):
    def get_bibliographies(self):
        # yield bibliography and its related source
        # XXX for now, only dbpedia provides bibliography details by lang
        if self.same_as:
            for ext_uri in self.related('same_as', 'object').entities():
                uri = '%s.rdf' % ext_uri.uri.replace('resource', 'data')
                try:
                    tree = etree.parse(uri)
                    lang = self._cw.lang
                    # try to get bibliography in application lang otherwise en
                    biblio = tree.xpath('/rdf:RDF/rdf:Description/dbpedia-owl:abstract[@xml:lang="%s"]' % lang,
                                        namespaces=NAMESPACES)
                    if biblio:
                        yield (unicode(biblio[0].text), ext_uri.uri)
                    else:
                        biblio = tree.xpath('/rdf:RDF/rdf:Description/dbpedia-owl:abstract[@xml:lang="en"]',
                                            namespaces=NAMESPACES)
                        if biblio:
                            yield (unicode(biblio[0].text), ext_uri.uri)
                        else:
                            continue

                except Exception, e:
                    print "EXCEPTION", e
                    continue




# -*- coding: utf-8 -*-
"""
Boxes for books

:organization: Logilab
:copyright: 2009-2017 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from cubicweb.selectors import is_instance, score_entity
from cubicweb.web.component import EntityCtxComponent

def has_isbn(entity):
    return entity.isbn13 is not None

class BookSeeAlso(EntityCtxComponent):
    __regid__ = 'book_seealso_box'
    __select__ = EntityCtxComponent.__select__ & is_instance('Book') & score_entity(has_isbn)
    context = 'navcontentbottom'

    order = 25 # display order of the box (25=display it last)

    def render_title(self, w):
        w(self._cw._('This book on other sites'))

    def render_body(self, w):
        entity = self.cw_rset.get_entity(0, 0)
        isbn = entity.isbn13 or u''
        self.append(self.link('OpenLibrary', 'http://openlibrary.org/isbn/%s' % isbn))
        self.append(self.link('Google Books', 'http://books.google.com/books?q=isbn:%s' % isbn))
        self.append(self.link('Amazon Books', 'http://www.amazon.com/gp/search/ref=sr_adv_b/?field-isbn=%s' % isbn))
        self.render_items(w)






"""
To use this script:
cubicweb-ctl shell appid import_dbpedia.py number_of_books_to_import

number_of_books_to_import is optionnal, if not provided, all books
will be imported
"""

import re

from lxml import etree

from cubicweb.dataimport import CWImportController, RQLObjectStore

from cubes.book.migration.import_ol import gen_books, to_date

NAMESPACES = {'res': "http://www.w3.org/2005/sparql-results#",
              'dbpprop': "http://dbpedia.org/property/",
              'foaf': "http://xmlns.com/foaf/0.1/",
              'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
              'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#"}

SPARQL = '''select ?book, ?title, ?pages, ?isbn, ?authors, ?publish_date, ?publisher where {
  ?book rdf:type <http://dbpedia.org/class/Book>.
  ?book dbpprop:name ?title.
  ?book dbpprop:pages ?pages.
  ?book dbpedia-owl:isbn ?isbn.
  ?book dbpedia-owl:author ?authors.
  ?book dbpprop:releaseDate ?publish_date.
  ?book dbpprop:publisher ?publisher
} %s'''

AUTHOR_SPARQL = '''select ?author where {
  ?author rdf:type <http://dbpedia.org/ontology/Person>.
  ?author foaf:name ?name
  FILTER regex(?name, "%s")
}'''

QUERY = '''http://dbpedia.org/sparql?query=%s&format=application/rdf+xml''' % SPARQL

def get_books(limit):
    if limit:
        limit = u'LIMIT %s' % limit
    else:
        limit = u''
    query = QUERY % limit
    tree = etree.parse(query)
    books = tree.xpath('/rdf:RDF/rdf:Description/res:solution',
                       namespaces=NAMESPACES)
    for book in books:
        try:
            bookinfo = {}
            for binding in book.xpath('res:binding', namespaces=NAMESPACES):
                attr = binding.xpath('res:variable', namespaces=NAMESPACES)[0].text
                # author uri is store in attribute rdf:resource of res:value
                if attr in ('authors', 'book'):
                    elt = binding.xpath('res:value', namespaces=NAMESPACES)[0]
                    value = unicode(elt.get('{%s}resource' % NAMESPACES['rdf']))
                elif attr == 'publisher':
                    # Publisher can either be an external resource or just plain text
                    elt = binding.xpath('res:value', namespaces=NAMESPACES)[0]
                    if elt.text:
                        value = unicode(elt.text)
                    else:
                        value = unicode(elt.get('{%s}resource' % NAMESPACES['rdf']))
                else:
                    value = unicode(binding.xpath('res:value', namespaces=NAMESPACES)[0].text)
                bookinfo[attr] = value
            print '----------', bookinfo['book']
            set_isbn(bookinfo)
            set_author(bookinfo)
            set_pages(bookinfo)
            set_publisher(bookinfo)
            bookinfo['cwuri'] = u'http://dbpedia.org'
            bookinfo['externaluri'] = bookinfo['book']
            bookinfo['publish_date'] = to_date(bookinfo['publish_date'])
            yield bookinfo
        except Exception, e:
            print "EXCEPTION in get_books:", e

def set_isbn(bookinfo):
    # isbn numbers follow the pattern:
    # ISBN x-xx-xxxxxx-x (xxxxx)
    #
    # there can be multiple isbn separated by a / or ,
    # This function selects the number in order to set
    # attributes isbn10 or isbn13 accordingly.
    # Or even sometimes, the value is the isbn itself or
    # just x-xx-xxxxxx-x.
    # We match here those four cases.
    initial_isbn = bookinfo['isbn']
    isbns = None
    if '/' in initial_isbn:
        isbns = initial_isbn.split('/')
    elif ',' in initial_isbn:
        isbns = initial_isbn.split(',')
    else:
        isbns = [initial_isbn]
    bookinfo['isbn10'] = bookinfo['isbn13'] = u''
    for item in isbns:
        matchs = re.match(r".*ISBN\s([^\s]+)", item)
        if matchs:
            isbn = matchs.group(1)
            num = isbn.replace('-', '').strip()
            if len(num) == 10:
                bookinfo['isbn10'] = num
            elif len(num) == 13:
                bookinfo['isbn13'] = num
            else:
                print 'Ignoring truncated ISBN number.'
        else:
            if '-' in item:
                item = item.replace('-', '')
            if len(item) == 10:
                bookinfo['isbn10'] = unicode(item)
            elif len(item) == 13:
                bookinfo['isbn13'] = unicode(item)
            else:
                print 'Ignoring truncated ISBN number.'

def set_author(bookinfo):
    author_uri = bookinfo['authors']
    url = '%s.rdf' % author_uri.replace('resource', 'data')
    try:
        tree = etree.parse(url)
        # try load foaf:name, if not found, then we try rdfs:label
        auth_name = tree.xpath('/rdf:RDF/rdf:Description/foaf:name',
                               namespaces=NAMESPACES) or \
                    tree.xpath('/rdf:RDF/rdf:Description/rdfs:label',
                                   namespaces=NAMESPACES)
        if len(auth_name) > 0 :
            auth_name = auth_name[0].text

        author_infos = {}
        author_infos['name'] = unicode(auth_name)
        author_infos['cwuri'] = u'http://dbpedia.org'
        author_infos['externaluri'] = unicode(author_uri)
        bookinfo['authors'] = [author_infos]
    except Exception, e:
        print "EXCEPTION in set_author:", e

def set_publisher(bookinfo):
    # Publisher is either a uri or plain text
    # In case a uri is specified, we will look for the corresponding
    # name in its rdf representation.
    publisher_uri = bookinfo['publisher']
    if publisher_uri.startswith('http'):
        url = '%s.rdf' % publisher_uri.replace('resource', 'data')
        try:
            tree = etree.parse(url)
            publisher_name = tree.xpath('/rdf:RDF/rdf:Description/rdfs:label',
                                        namespaces=NAMESPACES)
            if len(publisher_name) > 0 :
                bookinfo['publisher'] = unicode(publisher_name[0].text[:50])
            else:
                bookinfo['publisher'] = u''
        except Exception, e:
            print "EXCEPTION in set_publisher:", e


def set_pages(bookinfo):
    pages = bookinfo['pages']
    number = pages.split()[0]
    if number.isdigit():
        bookinfo['pages'] = number
    else:
        bookinfo['pages'] = None

GENERATORS = []
CHK = []
GENERATORS.append( (gen_books, CHK) )

if __name__ == '__main__':
    import sys
    try:
        limit = sys.argv[4]
    except:
        limit = None

    # create controller
    ctl = CWImportController(RQLObjectStore(session))
    ctl.askerror = 1
    ctl.generators = GENERATORS
    ctl.data['books'] = get_books(limit)
    # run
    ctl.run()



"""
Url rewriting for books.

:organization: Logilab
:copyright: 2009-2017 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from cubicweb.web.views.urlrewrite import SimpleReqRewriter, rgx

class BookReqRewriter(SimpleReqRewriter):
    rules = [
        (rgx(r'/book/(\d+)\.rdf'),
         dict(rql='Any B WHERE B is Book, B eid %(eid)s' % {'eid': r'\1'}, vid='bibo')),
        (rgx(r'/book/(\d+)\.html'),
         dict(rql='Any B WHERE B is Book, B eid %(eid)s' % {'eid': r'\1'})),
        (rgx(r'/book/isbn/(.*)'),
         dict(rql='Any B WHERE B is Book, (B isbn13 "%(isbn)s") OR (B isbn10 "%(isbn)s")' % {'isbn': r'\1'})),
        ]



# -*- coding: utf-8 -*-
"""
:organization: Logilab
:copyright: 2009-2017 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from datetime import datetime

from cubicweb import Binary, ValidationError
from cubicweb.selectors import is_instance
from cubicweb.server import hook

from cubes.book import olapi

def set_if_unset(entity, key, book, otherkey):
    if 'key' not in entity:
        if otherkey in book:
            value = book[otherkey]
            if isinstance(value, list):
                value = value[0]
            # NOTE: simplejson (v>=2.0.0) uses str for ascii strings
            entity[key] = unicode(value)


class BookCreationHook(hook.Hook):
    __regid__ = 'book_fetchinfo'
    __select__ = hook.Hook.__select__ & is_instance('Book')

    events = ('before_add_entity', )

    def __call__(self):
        entity = self.entity
        book = None
        if 'isbn13' in entity.cw_edited:
            try:
                book = olapi.get_bookinfo(isbn_13=entity.isbn13, cover_required=False)
            except olapi.OpenLibraryFailure:
                self._cw.info('failed to fetch book info from OpenLibrary for ISBN %s', entity.isbn13)

        elif 'isbn10' in entity.cw_edited:
            try:
                book = olapi.get_bookinfo(isbn_10=entity.isbn10, cover_required=False)
            except olapi.OpenLibraryFailure:
                self._cw.info('failed to fetch book info from OpenLibrary for ISBN %s', entity.isbn10)

        if not book:
            self.check_required_fields()
        else:
            languages = []
            for lang in book.get('languages', []):
                langinfo = olapi.openlibrary_query('/type/language', **lang)
                if langinfo:
                    langinfo = langinfo[0]
                languages.append(langinfo.get('name', '?'))
            if entity.language is None:
                entity['language'] =  u', '.join(languages)
            set_if_unset(entity, 'title', book, 'title')
            set_if_unset(entity, 'pages', book, 'number_of_pages')
            set_if_unset(entity, 'isbn10', book, 'isbn_10')
            if 'publish_date' in book:
                date = book['publish_date']
                try:
                    if ',' in date:
                        date = datetime.strptime(date, '%B %d, %Y')
                    elif date.isdigit():
                        date = datetime(int(date), 1, 1)
                    else:
                        raise ValueError('unknown date format')
                    if not entity.get('publish_date'):
                        entity['publish_date'] = date
                except ValueError:
                    self._cw.info('failed to parse publish_date %s', date)
            BookRelationPreCommit(self._cw, entity=entity, book=book)

    def check_required_fields(self):
        entity = self.entity
        if not entity.title:
            raise ValidationError(entity.eid,
                                  {'title':
                                   _('could not find the book in OpenLibrary. Please fill in the title and other fields you know')})

class BookRelationPreCommit(hook.Operation):

    def precommit_event(self):
        if not self.entity.has_cover and 'cover' in self.book:
            name = u'cover for %s' % self.book['title']
            self.session.execute('INSERT File F: F title %(n)s, F data %(d)s, '
                                 'F data_format %(f)s, F data_name %(dn)s, '
                                 'B has_cover F WHERE B eid %(x)s',
                                 {'n': name, 'x': self.entity.eid,
                                  'd': Binary(self.book['cover']),
                                  'f': u'image/jpeg',
                                  'dn': name})
        if not self.entity.publisher and 'publishers' in self.book:
            for publisher in self.book['publishers']:
                rset = self.session.execute('Any X WHERE X is Editor, X name %(name)s', {'name': unicode(publisher)})
                if rset:
                    for publisher in rset.entities(0):
                        publisher.set_relations(reverse_publisher=self.entity)
                else:
                    self.session.create_entity('Editor',
                                               name=unicode(publisher),
                                               reverse_publisher=self.entity)

        if not self.entity.authors and 'authors' in self.book:
            for author_info in olapi.get_authors(self.book):
                uri = u'http://openlibrary.org%s' % author_info['key']
                rset = self.session.execute('Any U WHERE U is ExternalUri, U uri %(uri)s', {'uri': uri})
                if rset:
                    author = rset.get_entity(0, 0).reverse_same_as[0]
                else:
                    author = self.session.create_entity('Person', surname=unicode(author_info['name']))
                author.set_relations(reverse_authors=self.entity)



"""Secondaries views for Book.

:organization: Logilab
:copyright: 2009-2017 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from logilab.mtconverter import xml_escape

from cubicweb.selectors import is_instance, score_entity
from cubicweb.view import EntityView
from cubicweb.uilib import cut


class BookListView(EntityView):
    __regid__ = 'listitem'
    __select__ = is_instance('Book')

    def cell_call(self, row, col, **kwargs):
        entity = self.cw_rset.get_entity(row, col)
        self.w(u'<div class="container clear">')
        self.w(u'<div class="left">')
        self.render_cover(entity)
        self.w(u'</div>')
        self.w(u'<div>')
        self.w(u'<h2><a href="%s">%s</a></h2>' % (entity.absolute_url(), xml_escape(entity.dc_title())))
        self.w(u'<p>%s - ' % (entity.printable_value('publish_date') or u''))
        self.w(u'%s' % self._cw._('by'))
        self.wview('csv', entity.related('authors'), 'null')
        self.w(u'</p></div></div><br />')

    def render_cover(self, entity):
        if entity.has_cover:
            imgs = [(image.absolute_url(vid='download'), image.data_name)
                    for image in entity.has_cover]
        else:
            imgs = [(self._cw.uiprops['PICTO_NOCOVER'],
                     self._cw._('no cover'))]
        for src, alt in imgs:
            self.w(u'<a href="%s">'
                   u'<img alt="%s" src="%s" width="50" style="align:right; width:30; height:50" />'
                   u'</a>' % (entity.absolute_url(), xml_escape(alt), xml_escape(src)))

class AuthorBibliographyView(EntityView):
    __regid__ = 'author-biblio'
    __select__ = is_instance('Person') & score_entity(lambda x: x.reverse_authors)

    def entity_call(self, entity):
        desc = cut(entity.dc_description(), 50)
        self.w(u'<a href="%s" title="%s">' % (
                xml_escape(entity.absolute_url()), xml_escape(desc)))
        self.w(xml_escape(self._cw.view('textincontext', entity=entity)))
        self.w(u'</a>')
        for (biblio, source) in entity.get_bibliographies():
            if source.startswith(u'http://dbpedia.org'):
                title = u'dbpedia'
            elif source.startswith(u'http://openlibrary.org'):
                title = u'openlibrary'
            else:
                title = self._cw._(u'unknown source')
            self.w(u'<div>%s<br/>(%s: <a href="%s">%s</a>)</div>' %
                    (biblio, self._cw._(u'Source'), source, self._cw._(title)))



from cubicweb.selectors import is_instance
from cubicweb.web.facet import RangeFacet, DateRangeFacet, HasRelationFacet, AttributeFacet

class BookPagesFacet(RangeFacet):
    __regid__ = 'priority-facet'
    __select__ = RangeFacet.__select__ & is_instance('Book')
    rtype = 'pages'

class BookPubdateFacet(DateRangeFacet):
    __regid__ = 'pubdate-facet'
    __select__ = DateRangeFacet.__select__ & is_instance('Book')
    rtype = 'publish_date'

class HasCoverFacet(HasRelationFacet):
    __regid__ = 'hascover-facet'
    __select__ = HasRelationFacet.__select__ & is_instance('Book')
    rtype = 'has_cover'

class CWUriFacet(AttributeFacet):
    __regid__ = 'cwuri-facet'
    __select__ = AttributeFacet.__select__ & is_instance('Book')
    rtype = 'cwuri'



"""
Primaries views for Book.

:organization: Logilab
:copyright: 2009-2017 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from logilab.mtconverter import xml_escape

from cubicweb.selectors import is_instance, score_entity
from cubicweb.web.views.primary import PrimaryView, RelatedView
from cubicweb.web import uicfg

uicfg.primaryview_section.tag_subject_of(('Book', 'has_cover', 'File'), 'hidden')
uicfg.primaryview_section.tag_subject_of(('Book', 'authors', '*'), 'hidden')
uicfg.primaryview_section.tag_subject_of(('Book', 'publisher', '*'), 'hidden')
uicfg.primaryview_section.tag_subject_of(('Book', 'editor', '*'), 'hidden')

uicfg.primaryview_section.tag_object_of(('Book', 'authors', 'Person'), 'relations')
uicfg.primaryview_section.tag_object_of(('Book', 'publisher', 'Editor'), 'relations')

uicfg.primaryview_section.tag_attribute(('Editor', 'name'), 'hidden')

class BookPrimaryView(PrimaryView):
    __select__ = is_instance('Book')

    def render_entity_title(self, entity):
        self.w(u'<h1>%s</h1>' % xml_escape(entity.dc_title()))

    def render_entity_attributes(self, entity):
        self._cw.add_css('cubes.book.css')
        self.w(u'<div class="container">')
        self.w(u'<div class="left">')
        self.render_cover(entity)
        self.w(u'</div>')
        self.w(u'<div>')
        self.w(u'<h3>')
        self.w(u'%s ' % self._cw._('by'))
        self.wview('csv', entity.related('authors'), 'null')
        self.w(u'</h3>')
        self.w(u'%s : %s' % (self._cw._('Summary'), xml_escape(entity.summary or u'')))
        self.w(u'</div>')
        self.w(u'</div>')
        self.render_details(entity)
        self.w(u'<div><h3>%s</h3></div>' % self._cw._('Content'))
        self.w(u'<div>%s</div>' % xml_escape(entity.content or u''))

    def render_details(self, entity):
        _ = self._cw._
        self.w(u'<div><h3>%s</h3>' % _('Details'))
        self.w(u'<ul>')
        self.w(u'<li> %s : %s </li>' % (_('Publication date'), entity.printable_value('publish_date') or u''))
        self.w(u'<li> %s : ' % _('Published by'))
        self.wview('oneline', entity.related('publisher'), 'null')
        self.w(u',  %s %s' % (entity.pages or u'?', _('pages')))
        self.w(u'</li>')
        self.w(u'<li> %s : ' % _('Editor'))
        self.wview('oneline', entity.related('editor'), 'null')
        self.w(u'</li>')
        self.w(u'<li> %s : ' % _('Collection'))
        self.wview('oneline', entity.related('collection'), 'null')
        self.w(u'</li>')
        self.w(u'<li> %s : %s' % (_('Language'), xml_escape(entity.language or u'')))
        self.w(u'</li>')
        self.w(u'<li> ISBN-10: %s' % xml_escape(entity.isbn10 or u''))
        self.w(u'</li>')
        self.w(u'<li>ISBN-13: %s' % xml_escape(entity.isbn13 or u''))
        self.w(u'</li>')
        self.w(u'</ul></div>')

    def render_cover(self, entity):
        if entity.has_cover:
            imgs = [(image.absolute_url(vid='download'), image.data_name)
                    for image in entity.has_cover]
        else:
            imgs = [(self._cw.uiprops['PICTO_NOCOVER'],
                     self._cw._('no cover'))]
        for src, alt in imgs:
            self.w(u'<img alt="%s" src="%s" style="align:right; width:110; height:130" />' %
                   (xml_escape(alt), xml_escape(src)))

class AuthorRelatedView(RelatedView):
    __select__ = is_instance('Person') & score_entity(lambda x: x.reverse_authors)

    def call(self, **kwargs):
        # nb: rset is retreived using entity.related with limit + 1 if any.
        # Because of that, we know that rset.printable_rql() will return rql
        # with no limit set anyway (since it's handled manually)
        if 'dispctrl' in self.cw_extra_kwargs:
            limit = self.cw_extra_kwargs['dispctrl'].get('limit')
            subvid = self.cw_extra_kwargs['dispctrl'].get('subvid', 'author-biblio')
        else:
            limit = None
            subvid = 'author-biblio'
        if limit is None or self.cw_rset.rowcount <= limit:
            if self.cw_rset.rowcount == 1:
                self.wview(subvid, self.cw_rset, row=0)
            elif 1 < self.cw_rset.rowcount <= 5:
                self.wview('author-biblio', self.cw_rset, subvid=subvid)
            else:
                self.w(u'<div>')
                self.wview('simplelist', self.cw_rset, subvid=subvid)
                self.w(u'</div>')
        # else show links to display related entities
        else:
            rql = self.cw_rset.printable_rql()
            self.cw_rset.limit(limit) # remove extra entity
            self.w(u'<div>')
            self.wview('simplelist', self.cw_rset, subvid=subvid)
            self.w(u'[<a href="%s">%s</a>]' % (
                xml_escape(self._cw.build_url(rql=rql, vid=subvid)),
                self._cw._('see them all')))
            self.w(u'</div>')



# pylint: disable-msg=W0622
"""cubicweb-book application packaging information"""

modname = 'book'
distname = 'cubicweb-%s' % modname

numversion = (0, 7, 1)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
description = 'component to describe books for the CubicWeb framework'
author = 'Logilab'
author_email = 'contact@logilab.fr'
long_desc = """
This cube provides the entity type ``Book`` and uses the OpenLibrary API_
to automatically fill the book's description

Check out : `Fetching book descriptions and covers`_

.. _`Fetching book descriptions and covers` : http://www.logilab.org/blogentry/9138
.. _API : http://openlibrary.org/dev/docs/api
"""
web = 'http://www.cubicweb.org/project/%s' % distname

classifiers = [
    'Environment :: Web Environment',
    'Framework :: CubicWeb',
    'Programming Language :: Python',
    'Programming Language :: JavaScript',
    ]

__depends__ = {'cubicweb': '>= 3.10.0',
               'cubicweb-addressbook': None,
               'cubicweb-person': None,
               'cubicweb-file': None,
               }

# packaging ###

from os import listdir as _listdir
from os.path import join, isdir
from glob import glob
scripts = glob(join('bin', 'book-*'))

THIS_CUBE_DIR = join('share', 'cubicweb', 'cubes', modname)

def listdir(dirpath):
    return [join(dirpath, fname) for fname in _listdir(dirpath)
            if fname[0] != '.' and not fname.endswith('.pyc')
            and not fname.endswith('~')
            and not isdir(join(dirpath, fname))]

data_files = [
    # common files
    [THIS_CUBE_DIR, [fname for fname in glob('*.py') if fname != 'setup.py']],
    ]
# check for possible extended cube layout
for dirname in ('entities', 'views', 'sobjects', 'hooks', 'schema', 'data', 'i18n',
                'migration', 'wdoc', 'data/images'):
    if isdir(dirname):
        data_files.append([join(THIS_CUBE_DIR, dirname), listdir(dirname)])
# Note: here, you'll need to add subdirectories if you want
# them to be included in the debian package




add_attribute('Book','pages')
add_attribute('Book','publish_date')
rset = rql('Any B,N WHERE B nbe_pages N')
for book, pages in rset:
    rql('SET B pages %(p)s WHERE B eid %(x)s', {'p':int(pages), 'x':book})
drop_attribute('Book', 'nbe_pages')
sync_schema_props_perms('Book')
checkpoint()



__docformat__ = "restructuredtext en"
_ = unicode

from logilab.mtconverter import xml_escape

from cubicweb.view import EntityView
from cubicweb.selectors import is_instance

class BiboView(EntityView):
    __regid__ = 'bibo'
    __select__ = is_instance('Book')

    title = _('bibo rdf')
    templatable = False
    content_type = 'application/rdf+xml'
    item_vid = 'bibo_item'

    def call(self):
        self.w(u'<?xml version="1.0" encoding="%s"?>\n' % self._cw.encoding)
        self.w(u'''<rdf:RDF
            xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
            xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
            xmlns:owl="http://www.w3.org/2002/07/owl#"
            xmlns:foaf="http://xmlns.com/foaf/0.1/"
            xmlns:bibo='http://purl.org/ontology/bibo/'
            xmlns:dcterms='http://purl.org/dc/terms/'
            xmlns:dc='http://purl.org/dc/elements/1.1/'
            >\n''')
        for i in xrange(self.cw_rset.rowcount):
            self.cell_call(row=i, col=0)
        self.w(u'</rdf:RDF>\n')

    def cell_call(self, row, col):
        self.wview(self.item_vid, self.cw_rset, row=row, col=col)



class BiboItemView(EntityView):
    __regid__ = 'bibo_item'
    __select__ = is_instance('Book')

    def cell_call(self, row, col):
        entity = self.cw_rset.complete_entity(row, col)
        self.w(u'<bibo:Book rdf:about="%s">\n' % xml_escape(entity.absolute_url()))
        self.w(u' <dc:title>%s</dc:title>' % xml_escape(entity.dc_title()))
        if entity.publish_date:
            self.w(u' <dc:date>%s</dc:date>' % xml_escape(entity.dc_date('%Y-%m-%d')))
        for same in entity.same_as:
            self.w(u' <owl:sameAs>%s</owl:sameAs>' % xml_escape(same.uri))
        if entity.isbn10:
            self.w(u' <bibo:isbn10>%s</bibo:isbn10>' % xml_escape(entity.isbn10))
        if entity.isbn13:
            self.w(u' <bibo:isbn13>%s</bibo:isbn13>' % xml_escape(entity.isbn13))
        for pub in entity.publisher:
            self.w(u' <dcterms:publisher>%s</dcterms:publisher>' % xml_escape(pub.dc_title()))
        self.w(u'</bibo:Book>\n')

